#include "u5shape_info.h"

#define MASK_GRASS_TO_GROUND		(0x01)
#define MASK_WATER_USE_MASK			(0x02)
#define MASK_FIRE_USE_MASK			(0x04)
#define MASK_ANIME_SCROLL			(0x08)
#define MASK_ANIME_FLAG				(0x10)
#define MASK_ANIME_2_FRAME			(0x20)
#define MASK_ANIME_4_FRAME			(0x40)

static const char shape_info_table[16][16] = {
	{	/* 0x0X*/
		0,
		MASK_ANIME_SCROLL,
		MASK_ANIME_SCROLL,
		MASK_ANIME_SCROLL,
		0,
		MASK_GRASS_TO_GROUND,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0
	},
	{	/* 0x1X */
		0,
		0,
		MASK_ANIME_FLAG,
		0,
		MASK_ANIME_FLAG,
		MASK_ANIME_FLAG,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		MASK_GRASS_TO_GROUND,
		MASK_GRASS_TO_GROUND
	},
	{	/* 0x2X */
		MASK_GRASS_TO_GROUND,
		MASK_GRASS_TO_GROUND,
		MASK_GRASS_TO_GROUND,
		MASK_GRASS_TO_GROUND,
		MASK_GRASS_TO_GROUND,
		MASK_GRASS_TO_GROUND,
		MASK_GRASS_TO_GROUND,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0
	},
	{	/* 0x3X */
		MASK_GRASS_TO_GROUND,
		MASK_GRASS_TO_GROUND,
		MASK_GRASS_TO_GROUND,
		MASK_GRASS_TO_GROUND,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		0,
		0,
		0,
		0,
		0,
		0,
		MASK_ANIME_FLAG,
		0
	},
	{	/* 0x4X */
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		MASK_GRASS_TO_GROUND,
#ifdef U5_PORT_APPLE2
		MASK_GRASS_TO_GROUND,		/* 0x4D */
#else
		0,							/* 0x4D */
#endif
		0,
		0
	},
	{	/* 0x5X */
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0
	},
	{	/* 0x6X */
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK,
		MASK_GRASS_TO_GROUND | MASK_WATER_USE_MASK
	},
	{	/* 0x7X */
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0
	},
	{	/* 0x8X */
		MASK_ANIME_2_FRAME,
		MASK_ANIME_2_FRAME,
		MASK_ANIME_2_FRAME,
		MASK_ANIME_2_FRAME,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		MASK_ANIME_SCROLL
	},
	{	/* 0x9X */
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0
	},
	{	/* 0xAX */
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0
	},
	{	/* 0xBX */
		MASK_FIRE_USE_MASK,
		MASK_FIRE_USE_MASK,
		MASK_FIRE_USE_MASK,
		MASK_FIRE_USE_MASK,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		MASK_FIRE_USE_MASK,
		MASK_FIRE_USE_MASK,
		MASK_FIRE_USE_MASK,
		MASK_FIRE_USE_MASK
	},
	{	/* 0xCX */
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0
	},
	{	/* 0xDX */
		0,
		0,
		0,
		0,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		0,
		0,
		MASK_FIRE_USE_MASK,
		0
	},
	{	/* 0xEX */
		0,
		0,
		0,
		0,
		MASK_WATER_USE_MASK,
		MASK_WATER_USE_MASK,
		MASK_WATER_USE_MASK,
		MASK_WATER_USE_MASK,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME,
		MASK_ANIME_4_FRAME
	},
	{	/* 0xFX */
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		MASK_ANIME_2_FRAME,
		MASK_ANIME_2_FRAME,
		MASK_ANIME_2_FRAME,
		MASK_ANIME_2_FRAME,
		0,
		0
	}
};

static const int mask_shape_table[16][16] = {
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x00D0, 0x00D1, 0x00D2, 0x00D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x0070, 0x0071, 0x0072, 0x0073, 0x0074, 0x0075, 0x0076, 0x0077, 0x0078, 0x0079, 0x007A, 0x007B, 0x007C, 0x007D, 0x007E, 0x007F },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x00C0, 0x00C1, 0x00C2, 0x00C3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x00CC, 0x00CD, 0x00CE, 0x00CF },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x00C2, 0x0000 },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x80D0, 0x80D1, 0x80D2, 0x80D3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 },
	{ 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }
};

int grass_to_ground(int row, int col)
{
	if (16 <= row) {
		return 0;
	}

	return (shape_info_table[row][col] & MASK_GRASS_TO_GROUND);
}

int get_water_mask(int row, int col)
{
	int mask_shape;

	if (16 <= row || (shape_info_table[row][col] & MASK_WATER_USE_MASK) == 0) {
		return 0;
	}

	mask_shape = mask_shape_table[row][col];
	if (mask_shape & 0x8000) {
		return -(mask_shape & 0x00FF);
	}
	return (mask_shape & 0x00FF);
}

int get_fire_mask(int row, int col)
{
	if (16 <= row || (shape_info_table[row][col] & MASK_FIRE_USE_MASK) == 0) {
		return 0;
	}

	return (mask_shape_table[row][col] & 0x00FF);
}

/* EOF */
